import { toast } from "sonner";

type ToastOptions<T> = {
    loading: () => string;
    success: (data: T) => string;
    error: (err: any) => string;
    finally?: () => void;
};

export function toasty<T>(
    promise: Promise<T>,
    options: ToastOptions<T>
): void {
    const loadingId = toast.loading(options.loading());

    promise
        .then((data: T) => {
            const result = data as any;

            if (result?.success === true) {
                toast.success(options.success?.(data) || "Success!", {
                    id: loadingId,
                });
            } else {
                toast.error(options.error?.(result) || result?.message || "Something went wrong", {
                    id: loadingId,
                });
            }
        })
        .catch((err: any) => {
            toast.error(options.error?.(err) || "An error occurred.", {
                id: loadingId,
            });
        })
        .finally(() => {
            options.finally?.();
        });
}
